@@ -6,7 +6,7 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
  */
 
 #ifndef NETLINK_LOCAL_TYPES_H_
@@ -30,7 +30,6 @@
 struct nl_cache_ops;
 struct nl_sock;
 struct nl_object;
-struct nl_hash_table;
 
 struct nl_cb
 {
@@ -70,7 +69,6 @@ struct nl_sock
 	unsigned int		s_seq_expect;
 	int			s_flags;
 	struct nl_cb *		s_cb;
-	size_t			s_bufsize;
 };
 
 struct nl_cache
@@ -79,9 +77,6 @@ struct nl_cache
 	int			c_nitems;
 	int                     c_iarg1;
 	int                     c_iarg2;
-	int			c_refcnt;
-	unsigned int		c_flags;
-	struct nl_hash_table *	hashtable;
 	struct nl_cache_ops *   c_ops;
 };
 
@@ -97,8 +92,7 @@ struct nl_cache_mngr
 	int			cm_protocol;
 	int			cm_flags;
 	int			cm_nassocs;
-	struct nl_sock *	cm_sock;
-	struct nl_sock *	cm_sync_sock;
+	struct nl_sock *	cm_handle;
 	struct nl_cache_assoc *	cm_assocs;
 };
 
@@ -173,16 +167,10 @@ struct rtnl_link
 	uint8_t				l_operstate;
 	uint8_t				l_linkmode;
 	/* 2 byte hole */
-	char *				l_info_kind;
 	struct rtnl_link_info_ops *	l_info_ops;
 	void *				l_af_data[AF_MAX];
 	void *				l_info;
 	char *				l_ifalias;
-	uint32_t			l_promiscuity;
-	uint32_t			l_num_tx_queues;
-	uint32_t			l_num_rx_queues;
-	uint32_t			l_group;
-	uint8_t				l_carrier;
 };
 
 struct rtnl_ncacheinfo
@@ -208,7 +196,6 @@ struct rtnl_neigh
 	struct rtnl_ncacheinfo n_cacheinfo;
 	uint32_t                n_state_mask;
 	uint32_t                n_flag_mask;
-	uint32_t		n_master;
 };
 
 
@@ -247,7 +234,6 @@ struct rtnl_addr
 	
 	char a_label[IFNAMSIZ];
 	uint32_t a_flag_mask;
-	struct rtnl_link *a_link;
 };
 
 struct rtnl_nexthop
@@ -524,7 +510,6 @@ struct rtnl_fw
 	struct nl_data *	cf_act;
 	struct nl_data *	cf_police;
 	char			cf_indev[IFNAMSIZ];
-	uint32_t		cf_fwmask;
 	int			cf_mask;
 };
 
@@ -645,7 +630,6 @@ struct rtnl_htb_qdisc
 	uint32_t		qh_rate2quantum;
 	uint32_t		qh_defcls;
 	uint32_t		qh_mask;
-	uint32_t		qh_direct_pkts;
 };
 
 struct rtnl_htb_class
@@ -657,7 +641,6 @@ struct rtnl_htb_class
 	uint32_t		ch_cbuffer;
 	uint32_t		ch_quantum;
 	uint32_t		ch_mask;
-	uint32_t		ch_level;
 };
 
 struct rtnl_cbq
@@ -682,12 +665,6 @@ struct rtnl_red
 	uint32_t	qr_mask;
 };
 
-struct rtnl_plug
-{
-	int             action;
-	uint32_t        limit;
-};
-
 struct flnl_request
 {
 	NLHDR_COMMON
@@ -791,50 +768,6 @@ struct nfnl_ct {
 	struct nfnl_ct_dir	ct_repl;
 };
 
-union nfnl_exp_protodata {
-	struct {
-		uint16_t	src;
-		uint16_t	dst;
-	} port;
-	struct {
-		uint16_t	id;
-		uint8_t		type;
-		uint8_t		code;
-	} icmp;
-};
-
-// Allow for different master/expect l4 protocols
-struct nfnl_exp_proto
-{
-	uint8_t						l4protonum;
-	union nfnl_exp_protodata	l4protodata;
-};
-
-struct nfnl_exp_dir {
-	struct nl_addr *		src;
-	struct nl_addr *		dst;
-	struct nfnl_exp_proto	proto;
-};
-
-struct nfnl_exp {
-	NLHDR_COMMON
-
-	uint8_t			exp_family;
-	uint32_t		exp_timeout;
-	uint32_t		exp_id;
-	uint16_t		exp_zone;
-	uint32_t		exp_class;
-	uint32_t		exp_flags;
-	char *			exp_helper_name;
-	char *			exp_fn;
-	uint8_t			exp_nat_dir;
-
-	struct nfnl_exp_dir		exp_expect;
-	struct nfnl_exp_dir		exp_master;
-	struct nfnl_exp_dir		exp_mask;
-	struct nfnl_exp_dir		exp_nat;
-};
-
 struct nfnl_log {
 	NLHDR_COMMON
 
