@@ -46,8 +46,9 @@ extern "C" {
 #define NLE_NOACCESS		27
 #define NLE_PERM		28
 #define NLE_PKTLOC_FILE		29
+#define NLE_PARSE_ERR		30
 
-#define NLE_MAX			NLE_PKTLOC_FILE
+#define NLE_MAX			NLE_PARSE_ERR
 
 extern const char *	nl_geterror(int);
 extern void		nl_perror(int, const char *);
