@@ -6,7 +6,7 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2013 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2006 Thomas Graf <tgraf@suug.ch>
  */
 
 #ifndef NETLINK_NETLINK_H_
@@ -16,7 +16,7 @@
 #include <stdint.h>
 #include <string.h>
 #include <stdlib.h>
-#include <poll.h>
+#include <sys/poll.h>
 #include <sys/socket.h>
 #include <sys/types.h>
 #include <sys/time.h>
@@ -26,23 +26,17 @@
 #include <linux/rtnetlink.h>
 #include <linux/genetlink.h>
 #include <linux/netfilter/nfnetlink.h>
-#include <netinet/tcp.h>
 #include <netlink/version.h>
 #include <netlink/errno.h>
 #include <netlink/types.h>
 #include <netlink/handlers.h>
 #include <netlink/socket.h>
-#include <netlink/object.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 struct ucred;
-struct nl_cache_ops;
-struct nl_parser_param;
-struct nl_object;
-struct nl_sock;
 
 extern int nl_debug;
 extern struct nl_dump_params nl_debug_dp;
@@ -65,7 +59,6 @@ extern void			nl_auto_complete(struct nl_sock *,
 extern int			nl_send_auto(struct nl_sock *, struct nl_msg *);
 extern int			nl_send_auto_complete(struct nl_sock *,
 						      struct nl_msg *);
-extern int			nl_send_sync(struct nl_sock *, struct nl_msg *);
 extern int			nl_send_simple(struct nl_sock *, int, int,
 					       void *, size_t);
 
@@ -75,23 +68,11 @@ extern int			nl_recv(struct nl_sock *,
 					struct ucred **);
 
 extern int			nl_recvmsgs(struct nl_sock *, struct nl_cb *);
-extern int			nl_recvmsgs_report(struct nl_sock *, struct nl_cb *);
 
 extern int			nl_recvmsgs_default(struct nl_sock *);
 
 extern int			nl_wait_for_ack(struct nl_sock *);
 
-extern int			nl_pickup(struct nl_sock *,
-					  int (*parser)(struct nl_cache_ops *,
-						struct sockaddr_nl *,
-						struct nlmsghdr *,
-						struct nl_parser_param *),
-					  struct nl_object **);
-extern int                      nl_pickup_keep_syserr(struct nl_sock *sk,
-                                                      int (*parser)(struct nl_cache_ops *, struct sockaddr_nl *,
-                                                                    struct nlmsghdr *, struct nl_parser_param *),
-                                                      struct nl_object **result,
-                                                      int *syserror);
 /* Netlink Family Translations */
 extern char *			nl_nlfamily2str(int, char *, size_t);
 extern int			nl_str2nlfamily(const char *);
