@@ -23,7 +23,7 @@ extern struct nl_sock *	nl_socket_alloc(void);
 extern struct nl_sock *	nl_socket_alloc_cb(struct nl_cb *);
 extern void		nl_socket_free(struct nl_sock *);
 
-extern uint32_t		nl_socket_get_local_port(struct nl_sock *);
+extern uint32_t		nl_socket_get_local_port(const struct nl_sock *);
 extern void		nl_socket_set_local_port(struct nl_sock *, uint32_t);
 
 extern int		nl_socket_add_memberships(struct nl_sock *, int, ...);
@@ -34,12 +34,12 @@ extern int		nl_socket_drop_membership(struct nl_sock *,
 extern void		nl_join_groups(struct nl_sock *, int);
 
 
-extern uint32_t		nl_socket_get_peer_port(struct nl_sock *);
+extern uint32_t		nl_socket_get_peer_port(const struct nl_sock *);
 extern void		nl_socket_set_peer_port(struct nl_sock *,
 							uint32_t);
-extern uint32_t 	nl_socket_get_peer_groups(struct nl_sock *sk);
+extern uint32_t 	nl_socket_get_peer_groups(const struct nl_sock *sk);
 extern void 		nl_socket_set_peer_groups(struct nl_sock *sk, uint32_t groups);
-extern struct nl_cb *	nl_socket_get_cb(struct nl_sock *);
+extern struct nl_cb *	nl_socket_get_cb(const struct nl_sock *);
 extern void		nl_socket_set_cb(struct nl_sock *,
 						 struct nl_cb *);
 extern int		nl_socket_modify_cb(struct nl_sock *, enum nl_cb_type,
@@ -55,8 +55,8 @@ extern unsigned int	nl_socket_use_seq(struct nl_sock *);
 extern void		nl_socket_disable_auto_ack(struct nl_sock *);
 extern void		nl_socket_enable_auto_ack(struct nl_sock *);
 
-extern int		nl_socket_get_fd(struct nl_sock *);
-extern int		nl_socket_set_nonblocking(struct nl_sock *);
+extern int		nl_socket_get_fd(const struct nl_sock *);
+extern int		nl_socket_set_nonblocking(const struct nl_sock *);
 extern void		nl_socket_enable_msg_peek(struct nl_sock *);
 extern void		nl_socket_disable_msg_peek(struct nl_sock *);
 
