@@ -6,7 +6,7 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2006 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
  */
 
 /**
@@ -31,7 +31,7 @@
 static struct nl_cache_ops genl_ctrl_ops;
 /** @endcond */
 
-static int ctrl_request_update(struct nl_cache *c, struct nl_handle *h)
+static int ctrl_request_update(struct nl_cache *c, struct nl_sock *h)
 {
 	return genl_send_simple(h, GENL_ID_CTRL, CTRL_CMD_GETFAMILY,
 				CTRL_VERSION, NLM_F_DUMP);
@@ -45,6 +45,7 @@ static struct nla_policy ctrl_policy[CTRL_ATTR_MAX+1] = {
 	[CTRL_ATTR_HDRSIZE]	= { .type = NLA_U32 },
 	[CTRL_ATTR_MAXATTR]	= { .type = NLA_U32 },
 	[CTRL_ATTR_OPS]		= { .type = NLA_NESTED },
+	[CTRL_ATTR_MCAST_GROUPS] = { .type = NLA_NESTED },
 };
 
 static struct nla_policy family_op_policy[CTRL_ATTR_OP_MAX+1] = {
@@ -52,6 +53,11 @@ static struct nla_policy family_op_policy[CTRL_ATTR_OP_MAX+1] = {
 	[CTRL_ATTR_OP_FLAGS]	= { .type = NLA_U32 },
 };
 
+static struct nla_policy family_grp_policy[CTRL_ATTR_MCAST_GRP_MAX+1] = {
+	[CTRL_ATTR_MCAST_GRP_NAME] = { .type = NLA_STRING },
+	[CTRL_ATTR_MCAST_GRP_ID]   = { .type = NLA_U32 },
+};
+
 static int ctrl_msg_parser(struct nl_cache_ops *ops, struct genl_cmd *cmd,
 			   struct genl_info *info, void *arg)
 {
@@ -61,17 +67,17 @@ static int ctrl_msg_parser(struct nl_cache_ops *ops, struct genl_cmd *cmd,
 
 	family = genl_family_alloc();
 	if (family == NULL) {
-		err = nl_errno(ENOMEM);
+		err = -NLE_NOMEM;
 		goto errout;
 	}
 
 	if (info->attrs[CTRL_ATTR_FAMILY_NAME] == NULL) {
-		err = nl_error(EINVAL, "Missing family name TLV");
+		err = -NLE_MISSING_ATTR;
 		goto errout;
 	}
 
 	if (info->attrs[CTRL_ATTR_FAMILY_ID] == NULL) {
-		err = nl_error(EINVAL, "Missing family id TLV");
+		err = -NLE_MISSING_ATTR;
 		goto errout;
 	}
 
@@ -111,7 +117,7 @@ static int ctrl_msg_parser(struct nl_cache_ops *ops, struct genl_cmd *cmd,
 				goto errout;
 
 			if (tb[CTRL_ATTR_OP_ID] == NULL) {
-				err = nl_errno(EINVAL);
+				err = -NLE_MISSING_ATTR;
 				goto errout;
 			}
 			
@@ -126,13 +132,42 @@ static int ctrl_msg_parser(struct nl_cache_ops *ops, struct genl_cmd *cmd,
 
 		}
 	}
+	
+	if (info->attrs[CTRL_ATTR_MCAST_GROUPS]) {
+		struct nlattr *nla, *nla_grps;
+		int remaining;
+
+		nla_grps = info->attrs[CTRL_ATTR_MCAST_GROUPS];
+		nla_for_each_nested(nla, nla_grps, remaining) {
+			struct nlattr *tb[CTRL_ATTR_MCAST_GRP_MAX+1];
+			int id;
+			const char * name;
+
+			err = nla_parse_nested(tb, CTRL_ATTR_MCAST_GRP_MAX, nla,
+					       family_grp_policy);
+			if (err < 0)
+				goto errout;
+
+			if (tb[CTRL_ATTR_MCAST_GRP_ID] == NULL) {
+				err = -NLE_MISSING_ATTR;
+				goto errout;
+			}
+			id = nla_get_u32(tb[CTRL_ATTR_MCAST_GRP_ID]);
+
+			if (tb[CTRL_ATTR_MCAST_GRP_NAME] == NULL) {
+				err = -NLE_MISSING_ATTR;
+				goto errout;
+			}
+			name = nla_get_string(tb[CTRL_ATTR_MCAST_GRP_NAME]);
+
+			err = genl_family_add_grp(family, id, name);
+			if (err < 0)
+				goto errout;
+		}
+
+	}
 
 	err = pp->pp_cb((struct nl_object *) family, pp);
-	if (err < 0)
-		goto errout;
-
-	err = P_ACCEPT;
-
 errout:
 	genl_family_put(family);
 	return err;
@@ -143,20 +178,9 @@ errout:
  * @{
  */
 
-struct nl_cache *genl_ctrl_alloc_cache(struct nl_handle *handle)
+int genl_ctrl_alloc_cache(struct nl_sock *sock, struct nl_cache **result)
 {
-	struct nl_cache * cache;
-	
-	cache = nl_cache_alloc(&genl_ctrl_ops);
-	if (cache == NULL)
-		return NULL;
-	
-	if (handle && nl_cache_refill(handle, cache) < 0) {
-		nl_cache_free(cache);
-		return NULL;
-	}
-
-	return cache;
+	return nl_cache_alloc_and_fill(&genl_ctrl_ops, sock, result);
 }
 
 /**
@@ -227,7 +251,7 @@ struct genl_family *genl_ctrl_search_by_name(struct nl_cache *cache,
 
 /**
  * Resolve generic netlink family name to its identifier
- * @arg handle		Netlink Handle
+ * @arg sk		Netlink socket.
  * @arg name		Name of generic netlink family
  *
  * Resolves the generic netlink family name to its identifer and returns
@@ -235,19 +259,18 @@ struct genl_family *genl_ctrl_search_by_name(struct nl_cache *cache,
  *
  * @return A positive identifier or a negative error code.
  */
-int genl_ctrl_resolve(struct nl_handle *handle, const char *name)
+int genl_ctrl_resolve(struct nl_sock *sk, const char *name)
 {
 	struct nl_cache *cache;
 	struct genl_family *family;
 	int err;
 
-	cache = genl_ctrl_alloc_cache(handle);
-	if (cache == NULL)
-		return nl_get_errno();
+	if ((err = genl_ctrl_alloc_cache(sk, &cache)) < 0)
+		return err;
 
 	family = genl_ctrl_search_by_name(cache, name);
 	if (family == NULL) {
-		err = nl_error(ENOENT, "Generic Netlink Family not found");
+		err = -NLE_OBJ_NOTFOUND;
 		goto errout;
 	}
 
@@ -259,6 +282,44 @@ errout:
 	return err;
 }
 
+static int genl_ctrl_grp_by_name(const struct genl_family *family,
+				const char *grp_name)
+{
+	struct genl_family_grp *grp;
+
+	nl_list_for_each_entry(grp, &family->gf_mc_grps, list) {
+		if (!strcmp(grp->name, grp_name)) {
+			return grp->id;
+		}
+	}
+
+	return 0;
+}
+
+int genl_ctrl_resolve_grp(struct nl_sock *sk, const char *family_name,
+	const char *grp_name)
+{
+	struct nl_cache *cache;
+	struct genl_family *family;
+	int err;
+
+	if ((err = genl_ctrl_alloc_cache(sk, &cache)) < 0)
+		return err;
+
+	family = genl_ctrl_search_by_name(cache, family_name);
+	if (family == NULL) {
+		err = -NLE_OBJ_NOTFOUND;
+		goto errout;
+	}
+
+	err = genl_ctrl_grp_by_name(family, grp_name);
+	genl_family_put(family);
+errout:
+	nl_cache_free(cache);
+
+	return err;
+}
+
 /** @} */
 
 static struct genl_cmd genl_cmds[] = {
