@@ -41,7 +41,7 @@ static int compare_id(const void *pa, const void *pb)
 
 	if (ma->classid < mb->classid)
 		return -1;
-
+	
 	if (ma->classid > mb->classid)
 		return 1;
 
@@ -106,7 +106,7 @@ static char *name_lookup(const uint32_t classid)
  * @return The destination buffer or the type encoded in hexidecimal
  *         form if no match was found.
  */
-char *rtnl_tc_handle2str(uint32_t handle, char *buf, size_t len)
+char * rtnl_tc_handle2str(uint32_t handle, char *buf, size_t len)
 {
 	if (TC_H_ROOT == handle)
 		snprintf(buf, len, "root");
@@ -120,11 +120,11 @@ char *rtnl_tc_handle2str(uint32_t handle, char *buf, size_t len)
 		if ((name = name_lookup(handle)))
 			snprintf(buf, len, "%s", name);
 		else if (0 == TC_H_MAJ(handle))
-			snprintf(buf, len, ":%x", TC_H_MIN(handle));
+			snprintf(buf, len, ":%02x", TC_H_MIN(handle));
 		else if (0 == TC_H_MIN(handle))
-			snprintf(buf, len, "%x:", TC_H_MAJ(handle) >> 16);
+			snprintf(buf, len, "%02x:", TC_H_MAJ(handle) >> 16);
 		else
-			snprintf(buf, len, "%x:%x",
+			snprintf(buf, len, "%02x:%02x",
 				TC_H_MAJ(handle) >> 16, TC_H_MIN(handle));
 	}
 
@@ -154,8 +154,7 @@ char *rtnl_tc_handle2str(uint32_t handle, char *buf, size_t len)
 int rtnl_tc_str2handle(const char *str, uint32_t *res)
 {
 	char *colon, *end;
-	uint32_t h;
-	int err;
+	uint32_t h, err;
 
 	if (!strcasecmp(str, "root")) {
 		*res = TC_H_ROOT;
@@ -167,11 +166,6 @@ int rtnl_tc_str2handle(const char *str, uint32_t *res)
 		return 0;
 	}
 
-	if (!strcasecmp(str, "ingress")) {
-		*res = TC_H_INGRESS;
-		return 0;
-	}
-
 	h = strtoul(str, &colon, 16);
 
 	/* MAJ is not a number */
@@ -223,7 +217,7 @@ not_a_number:
 		} else {
 			/* XXXX:YYYY */
 			uint32_t l;
-
+			
 update:
 			l = strtoul(colon+1, &end, 16);
 
@@ -302,7 +296,7 @@ static int classid_map_add(uint32_t classid, const char *name)
 
 /**
  * (Re-)read classid file
- *
+ * 
  * Rereads the contents of the classid file (typically found at the location
  * /etc/libnl/classid) and refreshes the classid maps.
  *
@@ -311,13 +305,12 @@ static int classid_map_add(uint32_t classid, const char *name)
 int rtnl_tc_read_classid_file(void)
 {
 	static time_t last_read;
-	struct stat st;
+	struct stat st = {0};
 	char buf[256], *path;
 	FILE *fd;
 	int err;
 
-	if (build_sysconf_path(&path, "classid") < 0)
-		return -NLE_NOMEM;
+	asprintf(&path, "%s/classid", SYSCONFDIR);
 
 	/* if stat fails, just (re-)read the file */
 	if (stat(path, &st) == 0) {
@@ -399,7 +392,7 @@ int rtnl_classid_generate(const char *name, uint32_t *result, uint32_t parent)
 
 	NL_DBG(2, "Generated new classid %#x\n", classid);
 
-	if (build_sysconf_path(&path, "classid") < 0)
+	if (asprintf(&path, "%s/classid", SYSCONFDIR) < 0)
 		return -NLE_NOMEM;
 
 	if (!(fd = fopen(path, "a"))) {
@@ -415,7 +408,7 @@ int rtnl_classid_generate(const char *name, uint32_t *result, uint32_t parent)
 	fclose(fd);
 
 	if ((err = classid_map_add(classid, name)) < 0) {
-		/*
+		/* 
 		 * Error adding classid map, re-read classid file is best
 		 * option here. It is likely to fail as well but better
 		 * than nothing, entry was added to the file already anyway.
@@ -441,16 +434,7 @@ static void __init classid_init(void)
 		nl_init_list_head(&tbl_name[i]);
 
 	if ((err = rtnl_tc_read_classid_file()) < 0)
-		NL_DBG(1, "Failed to read classid file: %s\n", nl_geterror(err));
+		fprintf(stderr, "Failed to read classid file: %s\n", nl_geterror(err));
 }
 
-static void free_map(void *map) {
-	free(((struct classid_map *)map)->name);
-	free(map);
-};
-
-static void __exit classid_exit(void)
-{
-	tdestroy(id_root, free_map);
-}
 /** @} */
