@@ -6,15 +6,15 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
  */
 
 /**
- * @defgroup rtnl Routing Library (libnl-route)
+ * @defgroup rtnl Routing Family
  * @{
  */
 
-#include <netlink-private/netlink.h>
+#include <netlink-local.h>
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
 #include <netlink/route/rtnl.h>
@@ -34,20 +34,15 @@
  * Fills out a routing netlink request message and sends it out
  * using nl_send_simple().
  *
- * @return 0 on success or a negative error code. Due to a bug in
- * older versions, this returned the number of bytes sent. So for
- * compatibility, treat positive return values as success too.
+ * @return 0 on success or a negative error code.
  */
 int nl_rtgen_request(struct nl_sock *sk, int type, int family, int flags)
 {
-	int err;
 	struct rtgenmsg gmsg = {
 		.rtgen_family = family,
 	};
 
-	err = nl_send_simple(sk, type, flags, &gmsg, sizeof(gmsg));
-
-	return err >= 0 ? 0 : err;
+	return nl_send_simple(sk, type, flags, &gmsg, sizeof(gmsg));
 }
 
 /** @} */
@@ -94,7 +89,7 @@ static const struct trans_tbl scopes[] = {
 	__ADD(254,host)
 	__ADD(253,link)
 	__ADD(200,site)
-	__ADD(0,global)
+	__ADD(0,universe)
 };
 
 char *rtnl_scope2str(int scope, char *buf, size_t size)
