@@ -6,29 +6,15 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
  */
 
 /**
- * @ingroup core_types
+ * @ingroup core
  * @defgroup socket Socket
- *
- * Representation of a netlink socket
- *
- * Related sections in the development guide:
- * - @core_doc{core_sockets, Netlink Sockets}
- *
  * @{
- *
- * Header
- * ------
- * ~~~~{.c}
- * #include <netlink/socket.h>
- * ~~~~
  */
 
-#include <pthread.h>
-
 #include <netlink-local.h>
 #include <netlink/netlink.h>
 #include <netlink/utils.h>
@@ -57,15 +43,12 @@ static void __init init_default_cb(void)
 }
 
 static uint32_t used_ports_map[32];
-static pthread_mutex_t port_map_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 static uint32_t generate_local_port(void)
 {
 	int i, n;
 	uint32_t pid = getpid() & 0x3FFFFF;
 
-	pthread_mutex_lock(&port_map_mutex);
-
 	for (i = 0; i < 32; i++) {
 		if (used_ports_map[i] == 0xFFFFFFFF)
 			continue;
@@ -79,15 +62,11 @@ static uint32_t generate_local_port(void)
 
 			/* PID_MAX_LIMIT is currently at 2^22, leaving 10 bit
 			 * to, i.e. 1024 unique ports per application. */
-
-			pthread_mutex_unlock(&port_map_mutex);
-
 			return pid + (n << 22);
+
 		}
 	}
 
-	pthread_mutex_unlock(&port_map_mutex);
-
 	/* Out of sockets in our own PID namespace, what to do? FIXME */
 	return UINT_MAX;
 }
@@ -100,10 +79,7 @@ static void release_local_port(uint32_t port)
 		return;
 	
 	nr = port >> 22;
-
-	pthread_mutex_lock(&port_map_mutex);
-	used_ports_map[nr / 32] &= ~(1 << (nr % 32));
-	pthread_mutex_unlock(&port_map_mutex);
+	used_ports_map[nr / 32] &= ~(1 << nr % 32);
 }
 
 /**
@@ -280,14 +256,7 @@ void nl_socket_set_local_port(struct nl_sock *sk, uint32_t port)
 {
 	if (port == 0) {
 		port = generate_local_port(); 
-		/*
-		 * Release local port after generation of a new one to be
-		 * able to change local port using nl_socket_set_local_port(, 0)
-		 */
-		if (!(sk->s_flags & NL_OWN_PORT))
-			release_local_port(sk->s_local.nl_pid);
-		else
-			sk->s_flags &= ~NL_OWN_PORT;
+		sk->s_flags &= ~NL_OWN_PORT;
 	} else  {
 		if (!(sk->s_flags & NL_OWN_PORT))
 			release_local_port(sk->s_local.nl_pid);
@@ -507,12 +476,12 @@ void nl_socket_set_cb(struct nl_sock *sk, struct nl_cb *cb)
 }
 
 /**
- * Modify the callback handler associated with the socket
+ * Modify the callback handler associated to the socket
  * @arg sk		Netlink socket.
  * @arg type		which type callback to set
  * @arg kind		kind of callback
  * @arg func		callback function
- * @arg arg		argument to be passed to callback function
+ * @arg arg		argument to be passwd to callback function
  *
  * @see nl_cb_set
  */
@@ -523,21 +492,6 @@ int nl_socket_modify_cb(struct nl_sock *sk, enum nl_cb_type type,
 	return nl_cb_set(sk->s_cb, type, kind, func, arg);
 }
 
-/**
- * Modify the error callback handler associated with the socket
- * @arg sk		Netlink socket.
- * @arg kind		kind of callback
- * @arg func		callback function
- * @arg arg		argument to be passed to callback function
- *
- * @see nl_cb_err
- */
-int nl_socket_modify_err_cb(struct nl_sock *sk, enum nl_cb_kind kind,
-			    nl_recvmsg_err_cb_t func, void *arg)
-{
-	return nl_cb_err(sk->s_cb, kind, func, arg);
-}
-
 /** @} */
 
 /**
@@ -586,36 +540,6 @@ int nl_socket_set_buffer_size(struct nl_sock *sk, int rxbuf, int txbuf)
 	return 0;
 }
 
-/**
- * Set default message buffer size of netlink socket.
- * @arg sk		Netlink socket.
- * @arg bufsize		Default message buffer size in bytes.
- *
- * Sets the default message buffer size to the specified length in bytes.
- * The default message buffer size limits the maximum message size the
- * socket will be able to receive. It is generally recommneded to specify
- * a buffer size no less than the size of a memory page.
- *
- * @return 0 on success or a negative error code.
- */
-int nl_socket_set_msg_buf_size(struct nl_sock *sk, size_t bufsize)
-{
-	sk->s_bufsize = bufsize;
-
-	return 0;
-}
-
-/**
- * Get default message buffer size of netlink socket.
- * @arg sk		Netlink socket.
- *
- * @return Size of default message buffer.
- */
-size_t nl_socket_get_msg_buf_size(struct nl_sock *sk)
-{
-	return sk->s_bufsize;
-}
-
 /**
  * Enable/disable credential passing on netlink socket.
  * @arg sk		Netlink socket.
